(***********************************************************)
(**  D-Sub 25 LPT Demo Example                                     **)
(***********************************************************)
(**  Created by: Ching-Yuh Huang   seaturn99@yahoo.com.tw **)
(**  Created on: 13/02/2004                               **)
(**                                                       **)
(**  ToDo : EPP,ECP implementation                        **)
(***********************************************************)
(** Licence:                                              **)
(***********************************************************)
(** iHHNק,Yקεo{ bug ,г· mail @ **)
(***********************************************************)

unit IEEE1284;

interface

uses Classes, SysUtils, zlportio;

const
  DEFAULT_INIT_BYTE = $04;

const
  STATUS_PORT_OFFSET = $1;
const
  CONTROL_PORT_OFFSET = $2;

const
  BIT0 = $01;
const
  BIT1 = $02;
const
  BIT2 = $04;
const
  BIT3 = $08;
const
  BIT4 = $10;
const
  BIT5 = $20;
const
  BIT6 = $40;
const
  BIT7 = $80;

const
  DATA_S0 = BIT0; // Reserved
const
  DATA_S1 = BIT1; // Reserved
const
  DATA_S2 = BIT2; // IRQ*
const
  DATA_S3 = BIT3; // FAULT*
const
  DATA_S4 = BIT4; // Select
const
  DATA_S5 = BIT5; // PError
const
  DATA_S6 = BIT6; // ACK*
const
  DATA_S7 = BIT7; // BUSY*

const
  STATUS_S0 = BIT0; // Reserved
const
  STATUS_S1 = BIT1; // Reserved
const
  STATUS_S2 = BIT2; // IRQ*
const
  STATUS_S3 = BIT3; // FAULT*
const
  STATUS_S4 = BIT4; // Select
const
  STATUS_S5 = BIT5; // PError
const
  STATUS_S6 = BIT6; // ACK*
const
  STATUS_S7 = BIT7; // BUSY*

const
  CONTROL_C0 = BIT0; // STROBE
const
  CONTROL_C1 = BIT1; // AUTOFD
const
  CONTROL_C2 = BIT2; // INT*
const
  CONTROL_C3 = BIT3; // SelectIn
const
  CONTROL_C4 = BIT4; // IRQEN
const
  CONTROL_C5 = BIT5; // Reserved
const
  CONTROL_C6 = BIT6; // Reserved
const
  CONTROL_C7 = BIT7; // Reserved

const
  SPP_S2_IRQ = STATUS_S2;
const
  SPP_S3_FAULT = STATUS_S3;
const
  SPP_S4_SELECT = STATUS_S4;
const
  SPP_S5_PERROR = STATUS_S5;
const
  SPP_S6_ACK = STATUS_S6;
const
  SPP_S7_BUSY = STATUS_S7;

const
  SPP_C0_STROBE = CONTROL_C0;
const
  SPP_C1_AUTOFD = CONTROL_C1;
const
  SPP_C2_INIT = CONTROL_C2;
const
  SPP_C3_SELECTIN = CONTROL_C3;
const
  SPP_C4_IRQEN = CONTROL_C4;

type
  SPP_DB = record
    REG_Data: Word;
    REG_Control: Word;
    REG_Status: Word;
    REG_BI_IControl: Word;
    REG_BI_IStatus: Word;
    REG_Reserved: Word;
  end;

  // EPP ECP Not implement

type
  EPP_DB = record
    REG_Data: Word;
    REG_Control: Word;
    REG_Status: Word;
  end;

type
  ECP_DB = record
    REG_Data: Word;
    REG_Control: Word;
    REG_Status: Word;
  end;

type
  IEEE1824_DB = record
    LPT_Name: string;
    SPP: SPP_DB;
    EPP: EPP_DB;
    ECP: ECP_DB;
  end;

type
  IEEE1284Mode = set of (SPP, EPP, ECP);

function isBitSet(const sByte, tByte: byte): boolean;

type
  TParallePortDevice = class
  private
    FDeviceMode: IEEE1284Mode;
    FDevice: IEEE1824_DB;
    FLPT_Names: array of string;
    FLPTCount: integer;
    FStarted: boolean;

    function GetLPTName(Index: integer): string;
    function GetDataAddrByMode(const Mode: IEEE1284Mode): Word;
    function GetCTRLAddrByMode(const Mode: IEEE1284Mode): Word;
    function GetStatusAddrByMode(const Mode: IEEE1284Mode): Word;

  public
    constructor Create;
    destructor Destroy; override;
    function GetLPTNames: TStrings;
    procedure SelectLPT(const LPTName: string);

    procedure InitDevice; overload;
    procedure InitDevive(const InitByte: Byte); overload;

    function GetCTRLReg:byte;
    function GetSTATUSReg:byte;
    function GetDATAReg:byte;

    procedure WriteToCTRL_REG(const CByte: Byte);
    procedure WriteToData_REG(const DByte: Byte);

    function RedaFromCTRL_REG: Byte;
    function RedaFromData_REG: Byte;
    function RedaFromStatus_REG: Byte;

    function SPP_IsBusy(): boolean;
    function SPP_IsFault(): boolean;

    function SPP_IsSTROBE(): boolean;
    function SPP_IsINIT(): boolean;
    function SPP_IsSelectIn(): boolean;
    function SPP_IsAutoFD(): boolean;

    property DeviceMode: IEEE1284Mode read FDeviceMode;
    property Device: IEEE1824_DB read FDevice;
    property Started: boolean read FStarted;
    property LPTCount: integer read FLPTCount;
    property LPT_Names[Index: integer]: string read GetLPTName;
  end;

implementation

uses mxarrays, Vic_Res;

function isBitSet(const sByte, tByte: byte): boolean;
begin
  Result := false;

  if (sByte and tByte) = tByte then
    Result := true;
end;

function GetLPTPortAddr_NT(const LPTName: string): Word;
var
  VicRes: TVicRes;
  i, j: integer;
begin
  Result := 0;

  try
    VicRes := TVicRes.Create;
    VicRes.SearchByHardwareKey('ACPI\PNP0401');

    for i := 1 to VicRes.Devices do
      if StrPos(PAnsiChar(VicRes.DeviceDescriptor[i]), PAnsiChar(LPTName)) <> nil
        then
      begin
        // Search For First Base I/O Address
        for j := 1 to 8 do
          if (VicRes.PortBase[j] <> $FFFF) and (VicRes.PortBase[j] <> 0) then
          begin
            Result := VicRes.PortBase[j];
            break;
          end;
      end;
  finally
    FreeAndNil(VicRes);
  end;
end;

//------------------------------------------------------

constructor TParallePortDevice.Create;
begin
  inherited;
  // Define
  FDeviceMode := [SPP];
  FStarted := ZLIOStarted;
  zliosetiopm(FALSE);
end;

destructor TParallePortDevice.Destroy;
begin
  // Direct IO;
  SetLength(FLPT_Names, 0);
  inherited;
end;

function TParallePortDevice.GetLPTName(Index: integer): string;
begin
  if (FLPTCount > 0) and (Index < FLPTCount) then
    Result := FLPT_Names[Index]
  else
    raise EArrayError.Create('Invalid Array Index');
end;

function TParallePortDevice.GetDataAddrByMode(const Mode: IEEE1284Mode): Word;
begin
  Result := 0;
  if Mode = [SPP] then
    Result := FDevice.SPP.REG_Data
  else if Mode = [EPP] then
    Result := FDevice.EPP.REG_Data
  else if Mode = [ECP] then
    Result := FDevice.ECP.REG_Data;

  if Result = 0 then
    raise Exception.Create('Can''''t Find Data Register Address!!');
end;

function TParallePortDevice.GetCTRLAddrByMode(const Mode: IEEE1284Mode): Word;
begin
  Result := 0;
  if Mode = [SPP] then
    Result := FDevice.SPP.REG_Control
  else if Mode = [EPP] then
    Result := FDevice.EPP.REG_Control
  else if Mode = [ECP] then
    Result := FDevice.ECP.REG_Control;

  if Result = 0 then
    raise Exception.Create('Can''''t Find Control Register Address!!');
end;

function TParallePortDevice.GetStatusAddrByMode(const Mode: IEEE1284Mode): Word;
begin
  Result := 0;
  if Mode = [SPP] then
    Result := FDevice.SPP.REG_Status
  else if Mode = [EPP] then
    Result := FDevice.EPP.REG_Status
  else if Mode = [ECP] then
    Result := FDevice.ECP.REG_Status;

  if Result = 0 then
    raise Exception.Create('Can''''t Find Status Register Address!!');
end;

// ----------------------------------------------------

function TParallePortDevice.GetLPTNames: TStrings;
var
  i: integer;
  VicRes: TVicRes;
begin
  Result := TStringList.Create;
  try
    VicRes := TVicRes.Create;

    VicRes.SearchByHardwareKey('ACPI\PNP0401');
    FLPTCount := VicRes.Devices;

    SetLength(FLPT_Names, FLPTCount);

    for i := 1 to VicRes.Devices do
      FLPT_Names[i - 1] := VicRes.DeviceDescriptor[i];
  finally
    FreeAndNil(VicRes);
  end;

  for i := 0 to FLPTCount - 1 do
    Result.Add(FLPT_Names[i]);
end;

procedure TParallePortDevice.SelectLPT(const LPTName: string);
begin
  if FDeviceMode = [SPP] then
  begin
    FDevice.LPT_Name := LPTName;
    FDevice.SPP.REG_Data := GetLPTPortAddr_NT(LPTName);
    FDevice.SPP.REG_Status := FDevice.SPP.REG_Data + STATUS_PORT_OFFSET;
    FDevice.SPP.REG_Control := FDevice.SPP.REG_Data + CONTROL_PORT_OFFSET;
  end
  else
    raise Exception.Create('DeviceMode Error!!');
end;

procedure TParallePortDevice.InitDevice;
begin
  InitDevive(DEFAULT_INIT_BYTE);
end;

procedure TParallePortDevice.InitDevive(const InitByte: Byte);
begin
  WriteToCTRL_REG(InitByte);
end;

procedure TParallePortDevice.WriteToCTRL_REG(const CByte: Byte);
var
  BaseAddr: Word;
begin
  BaseAddr := GetCTRLAddrByMode(FDeviceMode);
  PortWriteB(BaseAddr, CByte);
end;

procedure TParallePortDevice.WriteToData_REG(const DByte: Byte);
var
  BaseAddr: Word;
begin
  BaseAddr := GetDataAddrByMode(FDeviceMode);
  PortWriteB(BaseAddr, DByte);
end;

function TParallePortDevice.RedaFromCTRL_REG: byte;
var
  BaseAddr: Word;
begin
  BaseAddr := GetCTRLAddrByMode(FDeviceMode);
  Result := PortReadB(BaseAddr);
end;

function TParallePortDevice.RedaFromData_REG: byte;
var
  BaseAddr: Word;
begin
  BaseAddr := GetDataAddrByMode(FDeviceMode);
  Result := PortReadB(BaseAddr);
end;

function TParallePortDevice.RedaFromStatus_REG: byte;
var
  BaseAddr: Word;
begin
  BaseAddr := GetStatusAddrByMode(FDeviceMode);
  Result := PortReadB(BaseAddr);
end;

function TParallePortDevice.SPP_IsBusy(): boolean;
begin
  Result := isBitSet(GetSTATUSReg,SPP_S7_BUSY);
end;

function TParallePortDevice.SPP_IsFault(): boolean;
begin
  Result := isBitSet(GetSTATUSReg,SPP_S3_FAULT);
end;

function TParallePortDevice.SPP_IsSTROBE(): boolean;
begin
  Result := isBitSet(GetCTRLReg,SPP_C0_STROBE);
end;

function TParallePortDevice.SPP_IsINIT(): boolean;
begin
  Result := isBitSet(GetCTRLReg,IEEE1284.SPP_C2_INIT);
end;

function TParallePortDevice.SPP_IsSelectIn(): boolean;
begin
  Result := isBitSet(GetCTRLReg,SPP_C3_SELECTIN);
end;

function TParallePortDevice.SPP_IsAutoFD(): boolean;
begin
  Result := isBitSet(GetCTRLReg,SPP_C1_AUTOFD);
end;

function TParallePortDevice.GetCTRLReg:byte;
begin
  Result := PortReadB(GetCTRLAddrByMode(FDeviceMode));
end;

function TParallePortDevice.GetSTATUSReg:byte;
begin
  Result := PortReadB(GetStatusAddrByMode(FDeviceMode));
end;

function TParallePortDevice.GetDATAReg:byte;
begin
  Result := PortReadB(GetDataAddrByMode(FDeviceMode));
end;
end.

